@@ -1,12 +1,12 @@
 #if 0
 #
 # Utility for manipulating Book Type Field of Physical Format Descriptor
-# located in lead-in of DVD+RW media. This is 6th version. 2nd version
+# located in lead-in of DVD+RW media. This is 7th version. 2nd version
 # added initial support for 2nd generation DVD+RW drives. 3rd version
 # adds support for DVD+R unit settings. 4th version checks if the unit
 # is of RICOH design and reliably recognizes drive generation. 5th
 # version adds support for Benq derivatives. 6th version fixes problem
-# with USB connected units?
+# with USB connected units? 7th version adds support for BTC units.
 #
 # The code is in public domain.
 #
@@ -347,17 +347,106 @@ int benq (Scsi_Command &cmd,int action,int book,int gen=0)
   return mediainfo (cmd);
 }
 
+int btc (Scsi_Command &cmd,int action,int book,int gen=0)
+{ int err,obligatory=0;
+  const char *brand="";
+
+    switch (action)
+    {	case OPT_INQ|OPT_MEDIA:
+		break;
+	case OPT_INQ:
+	case OPT_INQ|OPT_UNIT:
+	case OPT_INQ|OPT_UNITRW:
+	case OPT_INQ|OPT_UNITR:
+		fprintf (stderr,":-( Can't inquiry unit settings, "
+				"you have to set booktype prior every "
+				"recording to be sure.\n");
+		return 1;
+		break;
+	case OPT_MEDIA:
+		if (profile!=0x1A && profile!=0x14 && profile!=13)
+		{   fprintf (stderr,":-( action is applicable to DVDRW only\n");
+		    break;
+		}
+
+		obligatory = (profile==0x1A)?0x92:0x32;
+		if (book!=obligatory && book!=0x01)
+		{   fprintf (stderr,":-( BookType#%02x is not applicable\n",book);
+		    break;
+		}
+
+		cmd[0]=0x1F;	// PREVENT/ALLOW MEDIA REMOVAL
+		cmd[4]=1;	// "Prevent"
+		cmd[5]=0;
+		if ((err=cmd.transport()))
+		{   sperror ("PREVENT MEDIA REMOVAL",err);
+		    break;
+		}
+
+		cmd[0]=0xFA;
+		cmd[2]=book;
+		cmd[10]=0xAA;
+		cmd[11]=0xFF;
+		if ((err=cmd.transport()))
+		    sperror ("BTC_FAh[2]",err);
+		else
+		    wait_for_unit (cmd);
+
+		cmd[0]=0x1E;	// PREVENT/ALLOW MEDIA REMOVAL
+		cmd[5]=0;
+		if ((err=cmd.transport()))
+		    return sperror ("ALLOW MEDIA REMOVAL",err),1;
+
+		break;
+	case OPT_UNIT:
+		if (profile==0x1B || profile==0x11) goto set_unitr;
+	case OPT_UNITRW:
+		fprintf (stderr,":-( Can't set RW unit settings, "
+				"manipulate media.\n");
+		return 1;
+		break;
+	case OPT_UNITR:
+	set_unitr:
+		// unit requires media in, so we can rely on profile...
+		if (profile==0x1B)  obligatory=0xA1, brand="+R";
+		else                obligatory=0x20, brand="-R";
+
+		if (book!=obligatory && book!=0x01)
+		{   fprintf (stderr,":-( BookType#%02x is not applicable\n",book);
+		    return 1;
+		}
+
+		cmd[0]=0xFA;
+		cmd[3]=book;
+		cmd[4]=1;
+		cmd[10]=0xAA;
+		cmd[11]=0xFF;
+		if ((err=cmd.transport()))
+		    return sperror ("BTC_FAh[3]",err),1;
+		else
+		    printf ("Unit was instructed to brand DVD%s as DVD%s\n",
+			    brand,book==0x01?"-ROM":brand);
+		return 0;
+		break;
+	default:
+		break;
+    }
+
+  return mediainfo (cmd);
+}
+
 int main(int argc,char *argv[])
 { Scsi_Command  cmd;
   unsigned char buf[128];
   int book=0,action=0,generation=0;
-  const char *brand="-*";
   int i,err;
 
     for(dev=NULL,i=1;i<argc;i++)
-    {	if	(!strncmp (argv[i],"-dvd-rom",7))   book=0x01, brand="-ROM";
-	else if (!strncmp (argv[i],"-dvd+rw",7))    book=0x92, brand="+RW";
-	else if (!strncmp (argv[i],"-dvd+r",6))     book=0xA1, brand="+R";
+    {	if	(!strncmp (argv[i],"-dvd-rom",7))   book=0x01;
+	else if (!strncmp (argv[i],"-dvd+rw",7))    book=0x92;
+	else if (!strncmp (argv[i],"-dvd+r",6))     book=0xA1;
+	else if (!strncmp (argv[i],"-dvd-rw",7))    book=0x32;
+	else if (!strncmp (argv[i],"-dvd-r",6))     book=0x20;
 	else if (!strncmp (argv[i],"-inq",4))       book=0xFF, action|=OPT_INQ;
 	else if (!strncmp (argv[i],"-unit+rw",8))   action&=~OPT_MASK, action|=OPT_UNITRW;
 	else if (!strncmp (argv[i],"-unit+r",7))    action&=~OPT_MASK, action|=OPT_UNITR;
@@ -467,8 +556,20 @@ int main(int argc,char *argv[])
 	if (!(err=cmd.transport()))
 	    return benq (cmd,action,book);
     }
+    else
+    {	// See if it's BTC design...
+	cmd[0]=0xFA;
+	cmd[4]=1;
+	cmd[10]=0xAA;
+	cmd[11]=0xFF;
+	if (!(err=cmd.transport()))
+	    return btc (cmd,action,book);
+	else if (ASC(err)==0x3A)
+	    return fprintf (stderr,":-( BTC_FAh: no media mounted\n"),1;
+    }
 
-    fprintf (stderr,"This program targets units of RICOH and BENQ designs.\n"
+    fprintf (stderr,"This program targets units of "
+		    "RICOH, BENQ and BTC designs.\n"
 		    "%s doesn't appear to be one. Exiting.\n",
 		    dev);
 
