@@ -1,5 +1,5 @@
 /*
- * DVDRW format 4.8 by Andy Polyakov <appro@fy.chalmers.se>.
+ * DVDRW format 4.9 by Andy Polyakov <appro@fy.chalmers.se>.
  *
  * Use-it-on-your-own-risk, GPL bless...
  *
@@ -51,6 +51,8 @@
  * - unify error reporting;
  * - permit for -lead-out even for blank DVD+RW, needed(?) for SANYO
  *   derivatives;
+ * 4.9:
+ * - permit for DVD-RW blank even if format descriptors are not present;
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -107,7 +109,7 @@ int main (int argc, char *argv[])
   pid_t		pid;
 
     fprintf (stderr,"* DVDRW format utility by <appro@fy.chalmers.se>, "
-		    "version 4.8.\n");
+		    "version 4.9.\n");
 
     for (i=1;i<argc;i++) {
 	if (*argv[i] == '-')
@@ -180,7 +182,7 @@ int main (int argc, char *argv[])
     } while (once--);
 
     len = formats[3];
-    if (len&7 || len<16)
+    if (len&7 || len<8)
 	fprintf (stderr,":-( allocation length isn't sane\n"),
 	exit(1);
 
@@ -211,8 +213,10 @@ int main (int argc, char *argv[])
     }
 
     if (i==len)
-	fprintf (stderr,":-( can't locate appropriate format descriptor\n"),
-	exit(1);
+    {	fprintf (stderr,":-( can't locate appropriate format descriptor\n");
+	if (blank)	i=0;
+	else		exit(1);
+    }
 
     capacity = 0;
     capacity |= formats[4+i+0], capacity <<= 8;
@@ -220,10 +224,10 @@ int main (int argc, char *argv[])
     capacity |= formats[4+i+2], capacity <<= 8;
     capacity |= formats[4+i+3];
 
-    if (mmc_profile==0x1A)
+    if (mmc_profile==0x1A)	// DVD+RW
 	fprintf (stderr,"* %.1fGB DVD+RW media detected.\n",
 			2048.0*capacity/1e9);
-    else
+    else			// DVD-RW
 	fprintf (stderr,"* %.1fGB DVD-RW media in %s mode detected.\n",
 			2048.0*capacity/1e9,
 			mmc_profile==0x13?"Restricted Overwrite":"Sequential");
@@ -265,11 +269,13 @@ int main (int argc, char *argv[])
 				"             and wipe the data.\n",
 				argv[0]);
 	    else
-		fprintf (stderr,"- you have the option to re-run %s with:\n"
+	    {	fprintf (stderr,"- you have the option to re-run %s with:\n",
+				argv[0]);
+		if (i) fprintf (stderr,
 				"  -force[=full] to enforce new format or mode transition\n"
-				"                and wipe the data;\n"
-				"  -blank[=full] to change to Sequential mode.\n",
-				argv[0]);    
+				"                and wipe the data;\n");
+		fprintf (stderr,"  -blank[=full] to change to Sequential mode.\n");
+	    }
 	    exit (0);
 	}
 	else if (cmd.umount())
