@@ -1,5 +1,5 @@
 /*
- * growisofs 5.18 by Andy Polyakov <appro@fy.chalmers.se>.
+ * growisofs 5.19 by Andy Polyakov <appro@fy.chalmers.se>.
  *
  * Use-it-on-your-own-risk, GPL bless...
  *
@@ -281,12 +281,22 @@
  * - skip count-down if no_tty_check is set;
  * - -use-the-force-luke=tracksize:size option by suggestion from K3b;
  * - Linux: fix for "Bad file descriptor" with DVD+RW kernel patch;
+ * 5.19:
+ * - IRIX: IRIX 6.x port is added;
+ * - Solaris: get rid of media reload, which made it possible to improve
+ *   volume manager experience as well;
+ * - address speed verification issues with NEC ND-2500 and Plextor
+ *   PX-708A;
+ * - make DVD-RAM work in "poor-man" mode;
+ * - average write speed report at the end of recording;
+ * - LG GSA-4081B fails to "SET STREAMING" with "LBA OUT OF RANGE" for
+ *   DVD+RW media, but not e.g. DVD-R;
  */
 #define PRINT_VERSION(cmd)	do {			\
     char *s=strrchr((cmd),'/');				\
     s ? s++ : (s=(cmd));				\
     printf ("* %.*sgrowisofs by <appro@fy.chalmers.se>,"\
-	    " version 5.18,\n",(int)(s-(cmd)),(cmd));	\
+	    " version 5.19-1,\n",(int)(s-(cmd)),(cmd));	\
 } while (0)
 
 #define _LARGEFILE_SOURCE 
@@ -537,8 +547,8 @@ char *setup_fds (char *device)
 	    setuid (uid);	/* drop all privileges	*/
 	    return ioctl_device=device;
 	case 0x1A:	/* DVD+RW			*/
-	    if (poor_man>0) goto open_poor_man;
 	case 0x12:	/* DVD-RAM			*/
+	    if (poor_man>0) goto open_poor_man;
 	    break;
 	default:
 	    fprintf (stderr,":-( %s: media is not recognized as "
@@ -638,11 +648,13 @@ char *setup_fds (char *device)
 	case 0x14:	/* DVD-RW Sequential		*/
 	case 0x1B:	/* DVD+R			*/
 	case 0x1A:	/* DVD+RW			*/
+	open_poor_man:
 	    poor_man = 1;
 	    out_fd = dup(ioctl_fd);
 	    setuid (uid);	/* drop all privileges	*/
 	    return ioctl_device=device;
 	case 0x12:	/* DVD-RAM			*/
+	    if (poor_man>0) goto open_poor_man;
 	    out_fd = dup(ioctl_fd);
 	    goto opened_rw;
 	default:
@@ -747,11 +759,13 @@ char *setup_fds (char *device)
 	case 0x14:	/* DVD-RW Sequential		*/
 	case 0x1B:	/* DVD+R			*/
 	case 0x1A:	/* DVD+RW			*/
+	open_poor_man:
 	    poor_man = 1;
 	    out_fd = dup(in_fd);	/* it's ignored in poor_man ... */
 	    setuid (uid);	/* drop all privileges	*/
 	    return ioctl_device=cam->device_path;
 	case 0x12:	/* DVD-RAM			*/
+	    if (poor_man>0) goto open_poor_man;
 	    break;
 	default:
 	    fprintf (stderr,":-( %s: media is not recognized as "
@@ -846,6 +860,7 @@ char *setup_fds (char *device)
     mmc_profile = get_mmc_profile (ioctl_handle);
     switch (mmc_profile&0xFFFF)
     {	case 0x11:	/* DVD-R			*/
+	case 0x12:	/* DVD-RAM			*/
 	case 0x13:	/* DVD-RW Restricted Overwrite	*/
 	case 0x14:	/* DVD-RW Sequential		*/
 	case 0x1B:	/* DVD+R			*/
@@ -858,8 +873,6 @@ char *setup_fds (char *device)
 	    setuid (uid);	/* drop all privileges	*/
 #endif
 	    return ioctl_device=device;
-	case 0x12:	/* DVD-RAM			*/
-	    break;
 	default:
 	    fprintf (stderr,":-( %s: media is not recognized as "
 			    "recordable DVD: %X\n",device,mmc_profile);
@@ -1000,7 +1013,7 @@ char *setup_fds (char *device)
      */
     sprintf (rscsi,"/dev/rscsi/c%xt%x%c%x",
 		    (m>>16)&0xFF,(m>>12)&0xF,'l',(m>>8)&0xF);
-    ioctl_handle = (void *)open64 (rscsi,O_RDONLY);
+    ioctl_handle = (void *)(long)open64 (rscsi,O_RDONLY);
     if (ioctl_handle == INVALID_HANDLE)
     {	fprintf (stderr,":-( unable to open(\"%s\",O_RDONLY): ",rscsi),
 	perror (NULL);
@@ -1022,11 +1035,11 @@ char *setup_fds (char *device)
     mmc_profile = get_mmc_profile (ioctl_handle);
     switch (mmc_profile&0xFFFF)
     {	case 0x11:	/* DVD-R			*/
+	case 0x12:	/* DVD-RAM			*/
 	case 0x13:	/* DVD-RW Restricted Overwrite	*/
 	case 0x14:	/* DVD-RW Sequential		*/
 	case 0x1B:	/* DVD+R			*/
 	case 0x1A:	/* DVD+RW			*/
-	case 0x12:	/* DVD-RAM			*/
 	    poor_man = 1;
 	    out_fd = dup(ioctl_fd);
 #if 0	    /* HP-UX requires root privileges upon SIOC_IO ioctl */
@@ -1081,6 +1094,132 @@ static int dup2_zero (int oldfd, int newfd)
 #define close	close_zero
 #define dup2	dup2_zero
 
+#elif defined(__sgi)
+
+#include <sys/attributes.h>
+#include <sys/param.h>
+#include <mediad.h>
+
+char *setup_fds (char *device)
+{ uid_t uid=getuid();
+  struct stat64 sb,sc;
+  char	hw_path[MAXPATHLEN],*s;
+  int	hw_len=sizeof(hw_path)-1;
+  int	bus=0,tgt=4,lun=0;	/* default config for O2 */
+  static char rscsi[64];
+
+    /*
+     * We might be entering as euid=root!
+     */
+    if ((in_fd = open64 (device,O_RDONLY)) < 0)
+    {	if (errno==EIO)	goto tray_might_be_open;
+	fprintf (stderr,":-( unable to open(\"%s\",O_RDONLY): ",device),
+	perror (NULL), exit (FATAL_START(errno));
+    }
+
+    if (fstat64 (in_fd,&sb) < 0)
+	fprintf (stderr,":-( unable to stat(\"%s\"): ",device),
+	perror (NULL), exit (FATAL_START(errno));
+
+    if (!S_ISCHR(sb.st_mode))
+    {	if (S_ISBLK(sb.st_mode) && !strncmp (device,"/dev/dsk",7))
+	{   fprintf (stderr,":-) you most likely want to use "
+			    "/dev/r%s instead!\n",device+5);
+	    if (isatty(0) && !dry_run) poll(NULL,0,5000);
+	}
+      open_rw:
+	setuid(uid);	/* drop all privileges	*/
+	close (in_fd);	/* reopen as mortal	*/
+	if ((in_fd = open64 (device,O_RDONLY)) < 0)
+	    fprintf (stderr,":-( unable to reopen(\"%s\",O_RDONLY): ",device),
+	    perror (NULL), exit (FATAL_START(errno));
+	if ((out_fd = open64 (device,O_RDWR)) < 0)
+	    fprintf (stderr,":-( unable to open(\"%s\",O_RDWR): ",device),
+	    perror (NULL), exit (FATAL_START(errno));
+      opened_rw:
+	poor_man = 0;
+	close (ioctl_fd);
+	ioctl_handle = INVALID_HANDLE;
+	setuid (uid);	/* drop all privileges	*/
+	return device;
+    }
+
+    /*
+     * Still as euid=root! But note that get_mmc_profile makes sure it's
+     * an MMC device, as it terminates the program if the unit doesn't
+     * reply to GET CONFIGURATON. In combination with following switch
+     * this means that if installed set-root-uid, growisofs grants
+     * access to DVD burner(s), but not to any other device.
+     */
+
+  tray_might_be_open:
+
+    if ((in_fd<0) ? attr_get (device,"_devname",hw_path,&hw_len,0) :
+		    attr_getf (in_fd,"_devname",hw_path,&hw_len,0) )
+	fprintf (stderr,":-( unable to obtain hw_path for \"%s\": ",device),
+	perror (NULL), exit (FATAL_START(errno));
+    if (hw_len>=sizeof(hw_path)) hw_len=sizeof(hw_path)-1; /* paranoia */
+    hw_path[hw_len]='\0';
+
+    if ((s=strstr(hw_path,"/scsi_ctlr/")))
+	sscanf (s,"/scsi_ctlr/%d/target/%d/lun/%d/",&bus,&tgt,&lun);
+
+    /* Make sure user is using /dev/rdsk/dksXdYlZvol... */
+    if ((s=strstr(hw_path,"/disk/volume/char"))==NULL)
+    {   if (lun)
+	    fprintf (stderr,":-( stick to /dev/rdsk/dks%dd%dl%dvol!\n",
+			    bus,tgt,lun);
+	else
+	    fprintf (stderr,":-( stick to /dev/rdsk/dks%dd%dvol!\n",
+			    bus,tgt);
+	exit(FATAL_START(EINVAL));
+    }
+
+    memcpy (s,"/scsi",6);
+    ioctl_handle = (void *)(long)open64 (hw_path,O_RDWR);
+    if (ioctl_handle == INVALID_HANDLE)
+	fprintf (stderr,":-( unable to open(\"%s\",O_RDWR): ",hw_path),
+	perror (NULL), exit (FATAL_START(errno));
+    if (fstat64 (ioctl_fd,&sc) < 0)
+	fprintf (stderr,":-( unable to stat(\"%s\"): ",hw_path),
+	perror (NULL), exit (FATAL_START(errno));
+    memcpy (s,"/disk/volume/char",6);
+
+    mmc_profile = get_mmc_profile (ioctl_handle);
+    switch (mmc_profile&0xFFFF)
+    {	case 0x11:	/* DVD-R			*/
+	case 0x13:	/* DVD-RW Restricted Overwrite	*/
+	case 0x14:	/* DVD-RW Sequential		*/
+	case 0x1B:	/* DVD+R			*/
+	case 0x1A:	/* DVD+RW			*/
+	open_poor_man:
+	    poor_man = 1;
+	    out_fd = dup (ioctl_fd);
+	    if (in_fd<0 && (in_fd=open64 (hw_path,O_RDONLY))<0)
+		/* hope for the best? */ ;
+	    setuid (uid);	/* drop all privileges	*/
+	    mediad_get_exclusiveuse (hw_path,"growisofs");
+	    if (mediad_last_error ()==RMED_NOERROR)
+		putenv ("MEDIAD_GOT_EXCLUSIVEUSE=");	/* kludge... */
+	    sprintf (rscsi,"/dev/scsi/sc%dd%dl%d",bus,tgt,lun);
+	    return ioctl_device=rscsi;	/* might be bogus... */
+	case 0x12:	/* DVD-RAM			*/
+	    /* Some of latest tentative IRIX releases seem to
+	     * implement DVD-RAM writing at dksc level, but I'm
+	     * not sure which one. So I just fall down to poor-man,
+	     * at least for now... */
+	    goto open_poor_man;
+	    break;
+	default:
+	    fprintf (stderr,":-( %s: media is not recognized as "
+			    "recordable DVD: %X\n",device,mmc_profile);
+	    exit (FATAL_START(EMEDIUMTYPE));
+    }
+    /* not actually reached */
+    setuid(uid);
+    goto open_rw;
+}
+
 #else
 #error "Unsupported OS"
 #endif
@@ -1088,7 +1227,7 @@ static int dup2_zero (int oldfd, int newfd)
 int setup_C_parm (char *C_parm,struct iso_primary_descriptor *descr)
 { int next_session=-1,profile=mmc_profile&0xFFFF;
 
-    if (!poor_man || profile==0x1A || profile==0x13)
+    if (!poor_man || profile==0x1A || profile==0x13 || profile==0x12)
     {	next_session = from_733(descr->volume_space_size);
 	/* pad to closest 32K boundary */
 	next_session += 15;
@@ -1315,7 +1454,9 @@ int builtin_dd (int infd,int outfd,off64_t outoff)
 	outoff += DVD_BLOCK;
     }
 
-    printf ("builtin_dd: %lld*2KB out\n",(outoff-startoff)/2048);
+    printf ("builtin_dd: %lld*2KB out @ average %.1fx1385KBps\n",
+	    (outoff-startoff)/2048,
+	    ((outoff-startoff)/(1024.0*1385.0))/(time(NULL)-progress->zero));
 out:
     { int saved_errno=errno;
 
@@ -1856,11 +1997,12 @@ int main (int argc, char *argv[])
 	if (!ioctl_device) ioctl_device=out_device;
 
 	switch (mmc_profile&0xFFFF)
-	{   case 0x1A:	/* DVD+RW			*/
-	    case 0x1B:	/* DVD+R			*/
-	    case 0x11:	/* DVD-R Sequential		*/
+	{   case 0x11:	/* DVD-R Sequential		*/
+	    case 0x12:	/* DVD-RAM			*/
 	    case 0x13:	/* DVD-RW Restricted Overwrite	*/
 	    case 0x14:	/* DVD-RW Sequential		*/
+	    case 0x1A:	/* DVD+RW			*/
+	    case 0x1B:	/* DVD+R			*/
 		break;
 	    default:
 		fprintf (stderr,":-( %s: mounted media doesn't appear to be "
