@@ -123,6 +123,13 @@ int media_reload (char *name=NULL,struct stat *sb=NULL)
 		cmd[5] = 0;
 		if (cmd.transport()) return 1;
 	    }
+#if defined(__sun) || defined(sun)
+	    else if (volmgt_running())
+	    {	setuid(getuid());
+		execl("/usr/bin/volrmmount","volrmmount","-i",name,NULL);
+		return 0;	// not normally reached
+	    }
+#endif
 	    else return 0;	// m-m-m-m! patched kernel:-)
 	}
 	{ Scsi_Command cmd;
@@ -214,7 +221,7 @@ int get_mmc_profile (void *fd)
     // see if it's blank media
     if ((disc_info[2]&3) == 0)	blank=0x10000;
 
-    if (profile != 0x1A && profile != 0x13)
+    if (profile != 0x1A && profile != 0x13 && profile != 0x12)
 	return blank|profile;
 
     cmd[0] = 0x23;	// READ FORMAT CAPACITIES
@@ -275,6 +282,18 @@ static unsigned int get_2k_capacity (Scsi_Command &cmd)
 	    else if (nwa<2048)	ret /= 2048/nwa;
 	    break;
 
+	case 0x12:		// DVD-RAM
+	    // As for the moment of this writing I don't format DVD-RAM.
+	    // Therefore I just pull formatted capacity for now...
+	    ret  = formats[4+0]<<24;
+	    ret |= formats[4+1]<<16;
+	    ret |= formats[4+2]<<8;
+	    ret |= formats[4+3];
+	    nwa = formats[4+5]<<16|formats[4+6]<<8|formats[4+7];
+	    if (nwa>2048)	ret *= nwa/2048;
+	    else if (nwa<2048)	ret /= 2048/nwa;
+	    break;
+
 	case 0x1B:		// DVD+R
 	case 0x11:		// DVD-R
 	case 0x14:		// DVD-RW Sequential
@@ -721,21 +740,59 @@ static unsigned char *pull_page2A (Scsi_Command &cmd)
   return page2A;
 }
 
+static int pull_velocity (Scsi_Command &cmd,unsigned char *d)
+{ unsigned int len;
+  int v,err;
+  class autofree perf;
+
+    if ((d[4]&2) == 0)	return -1;	// LG GCA-4040N:-(
+
+    len = (d[0]<<24|d[1]<<16|d[2]<<8|d[3])-4;
+    if (len%16)		return -1;
+
+    len += 8;
+    if (len == (8+16))
+    {	velocity = d[8+ 4]<<24|d[8+ 5]<<16|d[8+ 6]<<8|d[8+ 7];
+	v =        d[8+12]<<24|d[8+13]<<16|d[8+14]<<8|d[8+15];
+	if (v>velocity) velocity=v;	// CAV?
+    }
+    else	// ZCLV
+    { unsigned int n = (len-8)/16;
+      unsigned char *p;
+
+	perf = (unsigned char *)malloc (len);
+	if (perf == NULL)
+	    fprintf (stderr,":-( memory exhausted\n"),
+	    exit(FATAL_START(ENOMEM));
+
+	cmd[0]=0xAC;	// GET PERFORMANCE
+	cmd[1]=4;	// ask for "Overall Write Performance"
+	cmd[8]=n>>8;
+	cmd[9]=n;	// real number of descriptors
+	cmd[10]=0;	// ask for descriptor in effect
+	cmd[11]=0;
+	if ((err=cmd.transport(READ,perf,len)))
+	    sperror ("GET CURRENT PERFORMANCE",err),
+	    exit (FATAL_START(errno));
+
+	// Pick the highest speed...
+	for (p=perf+8,len-=8;len;p+=16,len-=16)
+	{   v=p[ 4]<<24|p[ 5]<<16|p[ 6]<<8|p[ 7];
+	    if (v > velocity) velocity = v;
+	    v=p[12]<<24|p[13]<<16|p[14]<<8|p[15];
+	    if (v > velocity) velocity = v;	// ZCAV?
+	}
+    }
+
+  return 0;
+}
+
 static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 			  unsigned char *page2A)
 { unsigned int len;
   int err;
   unsigned char d[8+16];
-  class autofree {
-    private:
-	unsigned char *ptr;
-    public:
-	autofree()			{ ptr=NULL; }
-	~autofree()			{ if (ptr) free(ptr); }
-	unsigned char *operator=(unsigned char *str)
-					{ return ptr=str; }
-	operator unsigned char *()	{ return ptr; }
-  } perf;
+  class autofree perf;
 
     cmd[0]=0xAC;		// GET PERFORMACE
     cmd[9]=1;			// start with one descriptor
@@ -805,25 +862,8 @@ static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 	    velocity = p[28]<<8|p[29];
 	}
 #endif
-	else
-	{   if ((d[4]&2) == 0)	// I could have checked length too...
-#if 1	// at least LG GCA-4040N returns empty descriptor:-(
-	    {	errp = -1;
+	else if ((errp = pull_velocity (cmd,d)) < 0)
 		goto emulated_err;
-	    }
-#else
-	    {	fprintf (stderr,":-( unit fails to report current write performance\n");
-		return -1;		// well, SET CD SPEED might work...
-	    }
-#endif
-	    page2A = NULL;	// skip over MODE SENSE#2A
-
-	    // I'm not sure if it works in all situations, e.g. with
-	    // ZCLV, but I have to start somewhere, so I bet on the 
-	    // "Start Performance" field, at least to start with...
-	    velocity = d[8+4]<<24|d[8+5]<<16|d[8+6]<<8|d[8+7];
-	    // Note that the value is expected to be perfectly sane!
-	}
 
 	if (speed_factor != 0.0)
 	{ int i,j=len-8,v,v0,v1,minv,closesti,closestv=0;
@@ -839,6 +879,9 @@ static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 	    // lower speed, so I have to watch out for it...
 	    if (!dvddash && minv>(2*ONEX))
 		speed_factor /= 2.4;
+	    // some units, e.g. NEC ND-2500, offer 2x as minimal speed...
+	    else if (minv>=(2*ONEX))
+		speed_factor /= 2.0;
 
 	    v0=(int)(speed_factor*minv + 0.5);
 	    for (closesti=0,minv=0x7fffffff,i=0;i<j;i+=16)
@@ -861,7 +904,7 @@ static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 #if 0
 	    memset(pd+8,0xFF,4);
 #else	    // I might have to copy the value from current descriptor...
-	    unsigned int cap=get_2k_capacity(cmd);
+	    unsigned int cap=get_2k_capacity(cmd)-1;
 
 	    pd[8]=cap>>24;
 	    pd[9]=cap>>16;
@@ -880,11 +923,10 @@ static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 		sperror ("SET STREAMING",err),
 		exit (FATAL_START(errno));
 
-	    if (page2A==NULL) continue;
-
-	    // unit couldn't reply to GET CURRENT PERFORMANCE, we have
-	    // no other choice but to pull page 2A. Thanks to Pioneer
-	    // for messing up this code...
+	    // I pull the Page 2A in either case, because unit might
+	    // have provided irrelevant information (Plextor). Not to
+	    // mention cases when it simply failed to reply to GET
+	    // CURRENT PERFORMANCE (Pioneer, LG).
 
 	    unsigned int plen  = (page2A[0]<<8|page2A[1]) + 2;
 
@@ -900,9 +942,12 @@ static int set_speed_B6h (Scsi_Command &cmd,unsigned int dvddash,
 	}
 	else if (targetv)
 	{   if (targetv!=velocity)
-		fprintf (stderr,":-( Failed to change write speed: %d->%d\n",
-				velocity,targetv),
+	    {	if (errp==0)	// check Page 2A for speed then...
+		{   errp=-1; continue;   }
+		fprintf (stderr,":-( Failed to change write speed: "
+				"%d->%d\n",velocity,targetv),
 		exit (FATAL_START(EINVAL));
+	    }
 	    break;
 	}
     } while (targetv);
@@ -985,16 +1030,7 @@ static int plusminus_pages_setup (Scsi_Command &cmd,int profile)
 { unsigned int   plen,hlen,dvddash;
   unsigned char  header[16],p32;
   int            err;
-  class autofree {
-    private:
-	unsigned char *ptr;
-    public:
-	autofree()			{ ptr=NULL; }
-	~autofree()			{ if (ptr) free(ptr); }
-	unsigned char *operator=(unsigned char *str)
-					{ return ptr=str; }
-	operator unsigned char *()	{ return ptr; }
-  } page2A;
+  class autofree page2A;
 
     switch (profile)
     {	case 0x1A:	// DVD+RW
@@ -1545,6 +1581,10 @@ pwrite64_t poor_mans_setup (void *fd,off64_t leadout)
 	    }
 	    else atsignals (minus_r_finalize);
 	    break;
+	case 0x12:	// DVD-RAM
+	    // exclusively for speed settings...
+	    plusminus_pages_setup (cmd,profile);
+	    break;
 	default:
 	    fprintf (stderr,":-( mounted media[%X] is not supported\n",profile),
 	    exit(FATAL_START(EMEDIUMTYPE));
@@ -1593,6 +1633,7 @@ int poor_man_rewritable (void *fd, void *buf)
   int		err,profile=mmc_profile&0xFFFF;
 
     if (profile!=0x13 &&	// not DVD-RW Restricted Overwrite
+	profile!=0x12 &&	// nor DVD-RAM
 	profile!=0x1A)		// nor DVD+RW
 	return 0;
 
